unit CreateDataModCustOrderLinkedDataViews;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ppEndUsr, ppComm, ppRelatv, ppProd, ppClass, ppReport,
  daDataModule, daSQL, daDataView, daQueryDataView, daDBBDE, ppDBPipe, ppTypes,
  ppModule, ppBands, ppCache, daIDE, ppCtrls, ppPrnabl, ppStrtch, ppSubRpt,
  ppParameter,
  ppDsIntf,
  daDataManager,
  daSQLBuilder;

type
  TForm1 = class(TForm)
    ppReport1: TppReport;
    ppDesigner1: TppDesigner;
    Button1: TButton;
    Button2: TButton;
    ppParameterList1: TppParameterList;
    ppHeaderBand1: TppHeaderBand;
    ppDetailBand1: TppDetailBand;
    ppSubReport1: TppSubReport;
    ppChildReport1: TppChildReport;
    ppTitleBand1: TppTitleBand;
    ppDetailBand2: TppDetailBand;
    ppDBText2: TppDBText;
    ppSummaryBand1: TppSummaryBand;
    ppDBText1: TppDBText;
    ppFooterBand1: TppFooterBand;
    ppGroup1: TppGroup;
    ppGroupHeaderBand1: TppGroupHeaderBand;
    ppGroupFooterBand1: TppGroupFooterBand;
    CheckBox1: TCheckBox;
    procedure CheckBox1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FDetailDataview: TdaDataview;
    FMasterDataview: TdaDataview;
    FDataViewsCreated: Boolean;

    procedure CreateDataViews;
    function CreateCustomers(aDataModule: TdaDatamodule): TppDBPipeline;
    function CreateOrders(aDataModule: TdaDatamodule): TppDBPipeline;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  ppDB;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FDataViewsCreated := False;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  ppDesigner1.ShowModal;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin

  if not(FDataViewsCreated) then
    begin
      CreateDataViews;

      FDataViewsCreated := True;
    end;

end;

procedure TForm1.CreateDataViews;
var
  lDataModule: TdaDataModule;
begin

  lDataModule := daGetDataModule(ppReport1);

  if lDataModule = nil then
    lDataModule := TdaDataModule.CreateForReport(ppReport1);

  ppReport1.DataPipeline := CreateCustomers(lDataModule);

  ppSubReport1.DataPipeline := CreateOrders(lDataModule);

end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin

  if CheckBox1.Checked then
    ppUnRegisterDesignModule(TdaDataManager, 'TppDesignerWindow')
  else
    ppRegisterDesignModule(TdaDataManager, 'TppDesignerWindow');

end;

function TForm1.CreateCustomers(aDataModule: TdaDatamodule): TppDBPipeline;
var
  lSQL: TdaSQL;
  lDataView: TdaBDEQueryDataView;
  lPipeline: TppDBPipeline;
  lSQLBuilder: TdaSQLBuilder;
begin

  // create BDEQueryDataView
  lDataView := TdaBDEQueryDataView.Create(aDataModule);
  lDataView.Name := 'Customers';
  lDataView.Parent := aDataModule;
  lDataView.Init;

  // set DBPipeline name and user name
  lPipeline := TppDBPipeline(lDataView.DataPipelines[0]);
  lPipeline.Name := 'plCustomers';
  lPipeline.UserName := 'plCustomers';

  FMasterDataview := lDataview;

  // configure the SQL object
  lSQL := lDataView.SQL;
  lSQL.DatabaseName := 'DBDemos';
  lSQL.DatabaseType := dtParadox;
  lSQL.SQLType      := sqBDELocal;
  lSQL.Session := lDataView.Session;
  lSQL.DataPipelineName := 'plCustomers';

  // build the SQL
  lSQLBuilder := TdaSQLBuilder.Create(lSQL);

  lSQLBuilder.SelectTables.Add('Customer');
  lSQLBuilder.SelectFields.Add('Customer', 'CustNo');
  lSQLBuilder.SelectFields.Add('Customer', 'Company');

   lSQLBuilder.ApplyUpdates;
   lSQLBuilder.Free;

  Result := lPipeline;

end;

function TForm1.CreateOrders(aDataModule: TdaDatamodule): TppDBPipeline;
var
  lSQL: TdaSQL;
  lDataView: TdaBDEQueryDataView;
  lPipeline: TppDBPipeline;
  lSQLBuilder: TdaSQLBuilder;
begin

  lDataView := TdaBDEQueryDataView.Create(aDataModule);
  lDataView.Name := 'Orders';
  lDataView.Parent := aDataModule;
  lDataView.Init;

  FDetailDataview := lDataview;

  {update pipeline}
  lPipeline := TppDBPipeline(lDataView.DataPipelines[0]);
  lPipeline.Name := 'plOrders';
  lPipeline.UserName := 'plOrders';

  lSQL := lDataview.SQL;
  lSQL.DatabaseName := 'DBDemos';
  lSQL.DatabaseType := dtParadox;
  lSQL.SQLType      := sqBDELocal;
  lSQL.Session := lDataView.Session;
  lSQL.DataPipelineName := 'plOrders';

    // build the SQL
  lSQLBuilder := TdaSQLBuilder.Create(lSQL);

  lSQLBuilder.SelectTables.Add('Orders');
  lSQLBuilder.SelectFields.Add('Orders', 'CustNo');
  lSQLBuilder.SelectFields.Add('Orders', 'OrderNo');
  lSQLBuilder.SelectFields.Add('Orders', 'AmountPaid');

  {need linking order bys}
  lSQLBuilder.OrderByFields.Add('Orders', 'CustNo');
  lSQLBuilder.OrderByFields.Add('Orders', 'AmountPaid');

  lSQLBuilder.ApplyUpdates;
  lSQLBuilder.Free;

  // link the detail to the master
  FDetailDataview.CreateLink(FMasterDataView, 'CustNo', 'CustNo');

  Result := lPipeline;

end;

end.
